@@ -19,6 +19,7 @@
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "SDL_anigif.h"
 #include "sdl_idf.h"
@@ -168,7 +169,10 @@ void AG_FreeSurfaces( AG_Frame* frames, int nFrames )
 	}
 }
 
-
+#if SDL_VERSION_ATLEAST(2,0,0)
+extern SDL_Surface *SDL_DisplayFormat(SDL_Surface * surface);
+extern SDL_Surface *SDL_DisplayFormatAlpha(SDL_Surface * surface);
+#endif
 
 int AG_ConvertSurfacesToDisplayFormat( AG_Frame* frames, int nFrames )
 {
@@ -181,8 +185,15 @@ int AG_ConvertSurfacesToDisplayFormat( AG_Frame* frames, int nFrames )
 		{
 			if ( frames[i].surface )
 			{
+#if SDL_VERSION_ATLEAST(2,0,0)
+				SDL_Surface* surface;
+				if (SDL_GetColorKey(frames[i].surface, NULL) == 0)
+					surface = SDL_DisplayFormatAlpha(frames[i].surface);
+				else
+					surface = SDL_DisplayFormat(frames[i].surface);
+#else
 				SDL_Surface* surface = (frames[i].surface->flags & SDL_SRCCOLORKEY) ? SDL_DisplayFormatAlpha(frames[i].surface) : SDL_DisplayFormat(frames[i].surface);
-
+#endif
 				if ( surface )
 				{
 					SDL_FreeSurface( frames[i].surface );
@@ -204,9 +215,20 @@ int AG_NormalizeSurfacesToDisplayFormat( AG_Frame* frames, int nFrames )
 
 	if ( nFrames > 0 && frames && frames[0].surface )
 	{
+#if SDL_VERSION_ATLEAST(2,0,0)
+		SDL_Surface* mainSurface;
+		int newDispose;
+		if (SDL_GetColorKey(frames[0].surface, NULL) == 0) {
+			mainSurface = SDL_DisplayFormatAlpha(frames[0].surface);
+			newDispose = AG_DISPOSE_RESTORE_BACKGROUND;
+		} else {
+			mainSurface = SDL_DisplayFormat(frames[0].surface);
+			newDispose = AG_DISPOSE_NONE;
+		}
+#else
 		SDL_Surface* mainSurface = (frames[0].surface->flags & SDL_SRCCOLORKEY) ? SDL_DisplayFormatAlpha(frames[0].surface) : SDL_DisplayFormat(frames[0].surface);
 		const int newDispose = (frames[0].surface->flags & SDL_SRCCOLORKEY) ? AG_DISPOSE_RESTORE_BACKGROUND : AG_DISPOSE_NONE;
-
+#endif
 		if ( mainSurface )
 		{
 			int i;
@@ -381,8 +403,11 @@ int AG_LoadGIF_RW( SDL_RWops* src, AG_Frame* frames, int maxFrames, int *loop)
 				goto done;
 
 			if ( gd->g89.transparent >= 0 )
+#if SDL_VERSION_ATLEAST(2,0,0)
+				SDL_SetColorKey( image, SDL_TRUE, gd->g89.transparent );
+#else
 				SDL_SetColorKey( image, SDL_SRCCOLORKEY, gd->g89.transparent );
-
+#endif
 			frames[iFrame].surface	= image;
 			frames[iFrame].x		= LM_to_uint(buf[0], buf[1]);
 			frames[iFrame].y		= LM_to_uint(buf[2], buf[3]);
@@ -693,7 +718,7 @@ static SDL_Surface* ReadImage( gifdata* gd, int len, int height, int cmapSize, u
 		return NULL;
 	}
 
-	image = SDL_AllocSurface( SDL_SWSURFACE, len, height, 8, 0, 0, 0, 0 );
+	image = SDL_CreateRGBSurface( SDL_SWSURFACE, len, height, 8, 0, 0, 0, 0 );
 
 	for ( i = 0; i < cmapSize; i++ )
 	{
