@@ -1,8 +1,33 @@
+#include <SDL.h>
+#if SDL_VERSION_ATLEAST(2,0,0)
+#include "SDL2_rotozoom.c"
+#else
 /*  
 
-SDL_rotozoom.c - rotozoomer, zoomer and shrinker for 32bit or 8bit surfaces
+SDL_rotozoom.c: rotozoomer, zoomer and shrinker for 32bit or 8bit surfaces
 
-LGPL (c) A. Schiffler
+Copyright (C) 2001-2012  Andreas Schiffler
+
+This software is provided 'as-is', without any express or implied
+warranty. In no event will the authors be held liable for any damages
+arising from the use of this software.
+
+Permission is granted to anyone to use this software for any purpose,
+including commercial applications, and to alter it and redistribute it
+freely, subject to the following restrictions:
+
+1. The origin of this software must not be misrepresented; you must not
+claim that you wrote the original software. If you use this software
+in a product, an acknowledgment in the product documentation would be
+appreciated but is not required.
+
+2. Altered source versions must be plainly marked as such, and must not be
+misrepresented as being the original software.
+
+3. This notice may not be removed or altered from any source
+distribution.
+
+Andreas Schiffler -- aschiffler at ferzkopp dot net
 
 */
 
@@ -63,14 +88,14 @@ Uint32 _colorkey(SDL_Surface *src)
 {
 	Uint32 key = 0; 
 #if (SDL_MINOR_VERSION == 3)
- 	SDL_GetColorKey(src, &key);
+	SDL_GetColorKey(src, &key);
 #else
- 	if (src) 
- 	{
- 		key = src->format->colorkey;
+	if (src) 
+	{
+		key = src->format->colorkey;
 	}
 #endif
- 	return key;
+	return key;
 }
 
 
@@ -264,28 +289,13 @@ Assumes dst surface was allocated with the correct dimensions.
 */
 int _zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy, int smooth)
 {
-	int x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, ex, ey, t1, t2, sstep, lx, ly;
-	tColorRGBA *c00, *c01, *c10, *c11, *cswap;
+	int x, y, sx, sy, ssx, ssy, *sax, *say, *csax, *csay, *salast, csx, csy, ex, ey, cx, cy, sstep, sstepx, sstepy;
+	tColorRGBA *c00, *c01, *c10, *c11;
 	tColorRGBA *sp, *csp, *dp;
-	int dgap;
+	int spixelgap, spixelw, spixelh, dgap, t1, t2;
 
 	/*
-	* Variable setup 
-	*/
-	if (smooth) {
-		/*
-		* For interpolation: assume source dimension is one pixel 
-		* smaller to avoid overflow on right and bottom edge.     
-		*/
-		sx = (int) (65536.0 * (float) (src->w - 1) / (float) dst->w);
-		sy = (int) (65536.0 * (float) (src->h - 1) / (float) dst->h);
-	} else {
-		sx = (int) (65536.0 * (float) src->w / (float) dst->w);
-		sy = (int) (65536.0 * (float) src->h / (float) dst->h);
-	}
-
-	/*
-	* Allocate memory for row increments 
+	* Allocate memory for row/column increments 
 	*/
 	if ((sax = (int *) malloc((dst->w + 1) * sizeof(Uint32))) == NULL) {
 		return (-1);
@@ -298,30 +308,55 @@ int _zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy,
 	/*
 	* Precalculate row increments 
 	*/
-	sp = csp = (tColorRGBA *) src->pixels;
-	dp = (tColorRGBA *) dst->pixels;
+	spixelw = (src->w - 1);
+	spixelh = (src->h - 1);
+	if (smooth) {
+		sx = (int) (65536.0 * (float) spixelw / (float) (dst->w - 1));
+		sy = (int) (65536.0 * (float) spixelh / (float) (dst->h - 1));
+	} else {
+		sx = (int) (65536.0 * (float) (src->w) / (float) (dst->w));
+		sy = (int) (65536.0 * (float) (src->h) / (float) (dst->h));
+	}
 
-	if (flipx) csp += (src->w-1);
-	if (flipy) csp += ((src->pitch/4)*(src->h-1));
+	/* Maximum scaled source size */
+	ssx = (src->w << 16) - 1;
+	ssy = (src->h << 16) - 1;
 
+	/* Precalculate horizontal row increments */
 	csx = 0;
 	csax = sax;
 	for (x = 0; x <= dst->w; x++) {
 		*csax = csx;
 		csax++;
-		csx &= 0xffff;
 		csx += sx;
+
+		/* Guard from overflows */
+		if (csx > ssx) { 
+			csx = ssx; 
+		}
 	}
+
+	/* Precalculate vertical row increments */
 	csy = 0;
 	csay = say;
 	for (y = 0; y <= dst->h; y++) {
 		*csay = csy;
 		csay++;
-		csy &= 0xffff;
 		csy += sy;
+
+		/* Guard from overflows */
+		if (csy > ssy) {
+			csy = ssy;
+		}
 	}
 
+	sp = (tColorRGBA *) src->pixels;
+	dp = (tColorRGBA *) dst->pixels;
 	dgap = dst->pitch - dst->w * 4;
+	spixelgap = src->pitch/4;
+
+	if (flipx) sp += spixelw;
+	if (flipy) sp += (spixelgap * spixelh);
 
 	/*
 	* Switch between interpolating and non-interpolating code 
@@ -331,40 +366,44 @@ int _zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy,
 		/*
 		* Interpolating Zoom 
 		*/
-
-		/*
-		* Scan destination 
-		*/
 		csay = say;
-		ly = 0;
 		for (y = 0; y < dst->h; y++) {
-			/*
-			* Setup color source pointers 
-			*/
-			c00 = csp;
-			c01 = csp;
-			c01++;	    
-			c10 = csp;
-			c10 += src->pitch/4;
-			c11 = c10;
-			c11++;
-			if (flipx) {
-				cswap = c00; c00=c01; c01=cswap;
-				cswap = c10; c10=c11; c11=cswap;
-			}
-			if (flipy) {
-				cswap = c00; c00=c10; c10=cswap;
-				cswap = c01; c01=c11; c11=cswap;
-			}
-
+			csp = sp;
 			csax = sax;
-			lx = 0;
 			for (x = 0; x < dst->w; x++) {
 				/*
-				* Draw and interpolate colors 
+				* Setup color source pointers 
 				*/
 				ex = (*csax & 0xffff);
 				ey = (*csay & 0xffff);
+				cx = (*csax >> 16);
+				cy = (*csay >> 16);
+				sstepx = cx < spixelw;
+				sstepy = cy < spixelh;
+				c00 = sp;
+				c01 = sp;
+				c10 = sp;
+				if (sstepy) {
+					if (flipy) {
+						c10 -= spixelgap;
+					} else {
+						c10 += spixelgap;
+					}
+				}
+				c11 = c10;
+				if (sstepx) {
+					if (flipx) {
+						c01--;
+						c11--;
+					} else {
+						c01++;
+						c11++;
+					}
+				}
+
+				/*
+				* Draw and interpolate colors 
+				*/
 				t1 = ((((c01->r - c00->r) * ex) >> 16) + c00->r) & 0xff;
 				t2 = ((((c11->r - c10->r) * ex) >> 16) + c10->r) & 0xff;
 				dp->r = (((t2 - t1) * ey) >> 16) + t1;
@@ -376,94 +415,82 @@ int _zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy,
 				dp->b = (((t2 - t1) * ey) >> 16) + t1;
 				t1 = ((((c01->a - c00->a) * ex) >> 16) + c00->a) & 0xff;
 				t2 = ((((c11->a - c10->a) * ex) >> 16) + c10->a) & 0xff;
-				dp->a = (((t2 - t1) * ey) >> 16) + t1;
-
+				dp->a = (((t2 - t1) * ey) >> 16) + t1;				
 				/*
-				* Advance source pointers 
+				* Advance source pointer x
 				*/
-				csax++;
-				if (*csax > 0)
-				{
-					sstep = (*csax >> 16);
-					lx += sstep;
-					if (flipx) sstep = -sstep;
-					if (lx <= src->w)
-					{
-						c00 += sstep;
-						c01 += sstep;
-						c10 += sstep;
-						c11 += sstep;
-					}
+				salast = csax;
+				csax++;				
+				sstep = (*csax >> 16) - (*salast >> 16);
+				if (flipx) {
+					sp -= sstep;
+				} else {
+					sp += sstep;
 				}
 
 				/*
-				* Advance destination pointer 
+				* Advance destination pointer x
 				*/
 				dp++;
 			}
-
 			/*
-			* Advance source pointer 
+			* Advance source pointer y
 			*/
+			salast = csay;
 			csay++;
-			if (*csay > 0)
-			{
-				sstep = (*csay >> 16);
-				ly += sstep;				
-				if (flipy) sstep = -sstep;
-				if (ly < src->h)
-				{
-					csp += (sstep * (src->pitch/4));
-				}
+			sstep = (*csay >> 16) - (*salast >> 16);
+			sstep *= spixelgap;
+			if (flipy) { 
+				sp = csp - sstep;
+			} else {
+				sp = csp + sstep;
 			}
 
 			/*
-			* Advance destination pointers 
+			* Advance destination pointer y
 			*/
 			dp = (tColorRGBA *) ((Uint8 *) dp + dgap);
 		}
 	} else {
-
 		/*
 		* Non-Interpolating Zoom 
-		*/
+		*/		
 		csay = say;
 		for (y = 0; y < dst->h; y++) {
-			sp = csp;
+			csp = sp;
 			csax = sax;
 			for (x = 0; x < dst->w; x++) {
 				/*
 				* Draw 
 				*/
 				*dp = *sp;
+
 				/*
-				* Advance source pointers 
+				* Advance source pointer x
 				*/
-				csax++;
-				if (*csax > 0)
-				{
-					sstep = (*csax >> 16);
-					if (flipx) sstep = -sstep;
-					sp += sstep;
-				}
+				salast = csax;
+				csax++;				
+				sstep = (*csax >> 16) - (*salast >> 16);
+				if (flipx) sstep = -sstep;
+				sp += sstep;
+
 				/*
-				* Advance destination pointer 
+				* Advance destination pointer x
 				*/
 				dp++;
 			}
 			/*
-			* Advance source pointer 
+			* Advance source pointer y
 			*/
+			salast = csay;
 			csay++;
-			if (*csay > 0)
-			{
-				sstep = (*csay >> 16) * (src->pitch/4);
-				if (flipy) sstep = -sstep;
-				csp += sstep;
-			}
+			sstep = (*csay >> 16) - (*salast >> 16);
+			sstep *= spixelgap;
+			if (flipy) sstep = -sstep;			
+			sp = csp + sstep;
 
 			/*
-			* Advance destination pointers 
+			* Advance destination pointer y
 			*/
 			dp = (tColorRGBA *) ((Uint8 *) dp + dgap);
 		}
@@ -534,6 +561,7 @@ int _zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 			csx -= dst->w;
 			(*csax)++;
 		}
+		(*csax) = (*csax) * (flipx ? -1 : 1);
 		csax++;
 	}
 	csy = 0;
@@ -545,6 +573,7 @@ int _zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 			csy -= dst->h;
 			(*csay)++;
 		}
+		(*csay) = (*csay) * (flipy ? -1 : 1);
 		csay++;
 	}
 
@@ -563,7 +592,7 @@ int _zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 			/*
 			* Advance source pointers 
 			*/
-			sp += (*csax) * (flipx ? -1 : 1);
+			sp += (*csax);
 			csax++;
 			/*
 			* Advance destination pointer 
@@ -573,7 +602,7 @@ int _zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 		/*
 		* Advance source pointer (for row) 
 		*/
-		csp += ((*csay) * src->pitch) * (flipy ? -1 : 1);
+		csp += ((*csay) * src->pitch);
 		csay++;
 
 		/*
@@ -747,7 +776,7 @@ void transformSurfaceY(SDL_Surface * src, SDL_Surface * dst, int cx, int cy, int
 	/*
 	* Clear surface to colorkey 
 	*/ 	
-	memset(pc, (unsigned char) (_colorkey(src) & 0xff), dst->pitch * dst->h);
+	memset(pc, (int)(_colorkey(src) & 0xff), dst->pitch * dst->h);
 	/*
 	* Iterate through destination surface 
 	*/
@@ -826,73 +855,73 @@ SDL_Surface* rotateSurface90Degrees(SDL_Surface* src, int numClockwiseTurns)
 	dst_ipr = dst->pitch / bpp;
 
 	switch(numClockwiseTurns) {
-		case 0: /* Make a copy of the surface */
-			{
-				/* Unfortunately SDL_BlitSurface cannot be used to make a copy of the surface
-				   since it does not preserve alpha. */
+	case 0: /* Make a copy of the surface */
+		{
+			/* Unfortunately SDL_BlitSurface cannot be used to make a copy of the surface
+			since it does not preserve alpha. */
 
-				if (src->pitch == dst->pitch) {
-					/* If the pitch is the same for both surfaces, the memory can be copied all at once. */
-					memcpy(dst->pixels, src->pixels, (src->h * src->pitch));
-				}
-				else
-				{
-					/* If the pitch differs, copy each row separately */
-					srcBuf = (Uint32*)(src->pixels); 
-					dstBuf = (Uint32*)(dst->pixels);
-					for (row = 0; row < src->h; row++) {
-						memcpy(dstBuf, srcBuf, dst->w * bpp);
-						srcBuf += src_ipr;
-						dstBuf += dst_ipr;
-					} /* end for(col) */
-				} /* end for(row) */
+			if (src->pitch == dst->pitch) {
+				/* If the pitch is the same for both surfaces, the memory can be copied all at once. */
+				memcpy(dst->pixels, src->pixels, (src->h * src->pitch));
 			}
-			break;
+			else
+			{
+				/* If the pitch differs, copy each row separately */
+				srcBuf = (Uint32*)(src->pixels); 
+				dstBuf = (Uint32*)(dst->pixels);
+				for (row = 0; row < src->h; row++) {
+					memcpy(dstBuf, srcBuf, dst->w * bpp);
+					srcBuf += src_ipr;
+					dstBuf += dst_ipr;
+				} /* end for(col) */
+			} /* end for(row) */
+		}
+		break;
 
-			/* rotate clockwise */
-		case 1: /* rotated 90 degrees clockwise */
-		 {
-			 for (row = 0; row < src->h; ++row) {
-				 srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
-				 dstBuf = (Uint32*)(dst->pixels) + (dst->w - row - 1);
-				 for (col = 0; col < src->w; ++col) {
-					 *dstBuf = *srcBuf;
-					 ++srcBuf;
-					 dstBuf += dst_ipr;
-				 } 
-				 /* end for(col) */
-			 } 
-			 /* end for(row) */
-		 }
-		 break;
+		/* rotate clockwise */
+	case 1: /* rotated 90 degrees clockwise */
+		{
+			for (row = 0; row < src->h; ++row) {
+				srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
+				dstBuf = (Uint32*)(dst->pixels) + (dst->w - row - 1);
+				for (col = 0; col < src->w; ++col) {
+					*dstBuf = *srcBuf;
+					++srcBuf;
+					dstBuf += dst_ipr;
+				} 
+				/* end for(col) */
+			} 
+			/* end for(row) */
+		}
+		break;
 
-		case 2: /* rotated 180 degrees clockwise */
-		 {
-			 for (row = 0; row < src->h; ++row) {
-				 srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
-				 dstBuf = (Uint32*)(dst->pixels) + ((dst->h - row - 1) * dst_ipr) + (dst->w - 1);
-				 for (col = 0; col < src->w; ++col) {
-					 *dstBuf = *srcBuf;
-					 ++srcBuf;
-					 --dstBuf;
-				 } 
-			 } 
-		 }
-		 break;
+	case 2: /* rotated 180 degrees clockwise */
+		{
+			for (row = 0; row < src->h; ++row) {
+				srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
+				dstBuf = (Uint32*)(dst->pixels) + ((dst->h - row - 1) * dst_ipr) + (dst->w - 1);
+				for (col = 0; col < src->w; ++col) {
+					*dstBuf = *srcBuf;
+					++srcBuf;
+					--dstBuf;
+				} 
+			} 
+		}
+		break;
 
-		case 3:
-		 {
-			 for (row = 0; row < src->h; ++row) {
-				 srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
-				 dstBuf = (Uint32*)(dst->pixels) + row + ((dst->h - 1) * dst_ipr);
-				 for (col = 0; col < src->w; ++col) {
-					 *dstBuf = *srcBuf;
-					 ++srcBuf;
-					 dstBuf -= dst_ipr;
-				 } 
-			 } 
-		 }
-		 break;
+	case 3:
+		{
+			for (row = 0; row < src->h; ++row) {
+				srcBuf = (Uint32*)(src->pixels) + (row * src_ipr);
+				dstBuf = (Uint32*)(dst->pixels) + row + ((dst->h - 1) * dst_ipr);
+				for (col = 0; col < src->w; ++col) {
+					*dstBuf = *srcBuf;
+					++srcBuf;
+					dstBuf -= dst_ipr;
+				} 
+			} 
+		}
+		break;
 	} 
 	/* end switch */
 
@@ -922,8 +951,8 @@ SDL_Surface* rotateSurface90Degrees(SDL_Surface* src, int numClockwiseTurns)
 
 */
 void _rotozoomSurfaceSizeTrig(int width, int height, double angle, double zoomx, double zoomy, 
-							  int *dstwidth, int *dstheight, 
-							  double *canglezoom, double *sanglezoom)
+	int *dstwidth, int *dstheight, 
+	double *canglezoom, double *sanglezoom)
 {
 	double x, y, cx, cy, sx, sy;
 	double radangle;
@@ -937,8 +966,8 @@ void _rotozoomSurfaceSizeTrig(int width, int height, double angle, double zoomx,
 	*canglezoom = cos(radangle);
 	*sanglezoom *= zoomx;
 	*canglezoom *= zoomx;
-	x = width / 2;
-	y = height / 2;
+	x = (double)(width / 2);
+	y = (double)(height / 2);
 	cx = *canglezoom * x;
 	cy = *canglezoom * y;
 	sx = *sanglezoom * x;
@@ -1338,8 +1367,8 @@ void zoomSurfaceSize(int width, int height, double zoomx, double zoomy, int *dst
 	/*
 	* Calculate target size 
 	*/
-	*dstwidth = (int) ((double) width * zoomx);
-	*dstheight = (int) ((double) height * zoomy);
+	*dstwidth = (int) floor(((double) width * zoomx) + 0.5);
+	*dstheight = (int) floor(((double) height * zoomy) + 0.5);
 	if (*dstwidth < 1) {
 		*dstwidth = 1;
 	}
@@ -1401,6 +1430,9 @@ SDL_Surface *zoomSurface(SDL_Surface * src, double zoomx, double zoomy, int smoo
 			0xff000000,  0x00ff0000, 0x0000ff00, 0x000000ff
 #endif
 			);
+		if (rz_src == NULL) {
+			return NULL;
+		}
 		SDL_BlitSurface(src, NULL, rz_src, NULL);
 		src_converted = 1;
 		is32bit = 1;
@@ -1434,8 +1466,15 @@ SDL_Surface *zoomSurface(SDL_Surface * src, double zoomx, double zoomy, int smoo
 	}
 
 	/* Check target */
-	if (rz_dst == NULL)
+	if (rz_dst == NULL) {
+		/*
+		* Cleanup temp surface 
+		*/
+		if (src_converted) {
+			SDL_FreeSurface(rz_src);
+		}		
 		return NULL;
+	}
 
 	/* Adjust for guard rows */
 	rz_dst->h = dstheight;
@@ -1509,19 +1548,23 @@ The input surface is not modified. The output surface is newly allocated.
 
 \return The new, shrunken surface.
 */
+/*@null@*/ 
 SDL_Surface *shrinkSurface(SDL_Surface *src, int factorx, int factory)
 {
+	int result;
 	SDL_Surface *rz_src;
-	SDL_Surface *rz_dst;
+	SDL_Surface *rz_dst = NULL;
 	int dstwidth, dstheight;
 	int is32bit;
 	int i, src_converted;
+	int haveError = 0;
 
 	/*
 	* Sanity check 
 	*/
-	if (src == NULL)
+	if (src == NULL) {
 		return (NULL);
+	}
 
 	/*
 	* Determine if source surface is 32bit or 8bit 
@@ -1537,19 +1580,33 @@ SDL_Surface *shrinkSurface(SDL_Surface *src, int factorx, int factory)
 		/*
 		* New source surface is 32bit with a defined RGBA ordering 
 		*/
-		rz_src =
-			SDL_CreateRGBSurface(SDL_SWSURFACE, src->w, src->h, 32, 
+		rz_src = SDL_CreateRGBSurface(SDL_SWSURFACE, src->w, src->h, 32, 
 #if SDL_BYTEORDER == SDL_LIL_ENDIAN
 			0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000
 #else
 			0xff000000,  0x00ff0000, 0x0000ff00, 0x000000ff
 #endif
 			);
+		if (rz_src==NULL) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}
+
 		SDL_BlitSurface(src, NULL, rz_src, NULL);
 		src_converted = 1;
 		is32bit = 1;
 	}
 
+	/*
+	* Lock the surface 
+	*/
+	if (SDL_MUSTLOCK(rz_src)) {
+		if (SDL_LockSurface(rz_src) < 0) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}
+	}
+
 	/* Get size for target */
 	dstwidth=rz_src->w/factorx;
 	while (dstwidth*factorx>rz_src->w) { dstwidth--; }
@@ -1560,8 +1617,7 @@ SDL_Surface *shrinkSurface(SDL_Surface *src, int factorx, int factory)
 	* Alloc space to completely contain the shrunken surface
 	* (with added guard rows)
 	*/
-	rz_dst = NULL;
-	if (is32bit) {
+	if (is32bit==1) {
 		/*
 		* Target surface is 32bit with source RGBA/ABGR ordering 
 		*/
@@ -1575,33 +1631,37 @@ SDL_Surface *shrinkSurface(SDL_Surface *src, int factorx, int factory)
 		*/
 		rz_dst = SDL_CreateRGBSurface(SDL_SWSURFACE, dstwidth, dstheight + GUARD_ROWS, 8, 0, 0, 0, 0);
 	}
-	
+
 	/* Check target */
-	if (rz_dst == NULL)
-		return NULL;
+	if (rz_dst == NULL) {
+		haveError = 1;
+		goto exitShrinkSurface;
+	}
 
 	/* Adjust for guard rows */
 	rz_dst->h = dstheight;
 
-	/*
-	* Lock source surface 
-	*/
-	if (SDL_MUSTLOCK(rz_src)) {
-		SDL_LockSurface(rz_src);
-	}
-
 	/*
 	* Check which kind of surface we have 
 	*/
-	if (is32bit) {
+	if (is32bit==1) {
 		/*
 		* Call the 32bit transformation routine to do the shrinking (using alpha) 
 		*/
-		_shrinkSurfaceRGBA(rz_src, rz_dst, factorx, factory);
+		result = _shrinkSurfaceRGBA(rz_src, rz_dst, factorx, factory);		
+		if ((result!=0) || (rz_dst==NULL)) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}
+
 		/*
 		* Turn on source-alpha support 
 		*/
-		SDL_SetAlpha(rz_dst, SDL_SRCALPHA, 255);
+		result = SDL_SetAlpha(rz_dst, SDL_SRCALPHA, 255);
+		if (result!=0) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}
 	} else {
 		/*
 		* Copy palette and colorkey info 
@@ -1613,26 +1673,50 @@ SDL_Surface *shrinkSurface(SDL_Surface *src, int factorx, int factory)
 		/*
 		* Call the 8bit transformation routine to do the shrinking 
 		*/
-		_shrinkSurfaceY(rz_src, rz_dst, factorx, factory);
-		SDL_SetColorKey(rz_dst, SDL_SRCCOLORKEY | SDL_RLEACCEL, _colorkey(rz_src));
+		result = _shrinkSurfaceY(rz_src, rz_dst, factorx, factory);
+		if (result!=0) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}
+
+		/*
+		* Set colorkey on target
+		*/
+		result = SDL_SetColorKey(rz_dst, SDL_SRCCOLORKEY | SDL_RLEACCEL, _colorkey(rz_src));
+		if (result!=0) {
+			haveError = 1;
+			goto exitShrinkSurface;
+		}		
 	}
 
-	/*
-	* Unlock source surface 
-	*/
-	if (SDL_MUSTLOCK(rz_src)) {
-		SDL_UnlockSurface(rz_src);
+exitShrinkSurface:
+	if (rz_src!=NULL) {
+		/*
+		* Unlock source surface 
+		*/
+		if (SDL_MUSTLOCK(rz_src)) {
+			SDL_UnlockSurface(rz_src);
+		}
+
+		/*
+		* Cleanup temp surface 
+		*/
+		if (src_converted==1) {
+			SDL_FreeSurface(rz_src);
+		}
 	}
 
-	/*
-	* Cleanup temp surface 
-	*/
-	if (src_converted) {
-		SDL_FreeSurface(rz_src);
-	}
+	/* Check error state; maybe need to cleanup destination */
+	if (haveError==1) {
+		if (rz_dst!=NULL) {
+			SDL_FreeSurface(rz_dst);
+		}
+		rz_dst=NULL;
+	} 
 
 	/*
 	* Return destination surface 
 	*/
 	return (rz_dst);
 }
+#endif
