@@ -10,9 +10,15 @@
 #define KBD_INVERSE	2
 #define KBD_MAX		3
 
+#define JUST_THEME	0
+#define JUST_NO		1
+#define JUST_YES	2
+#define JUST_MAX	3
+
 extern char *appdata_sw;
 extern int chunksize_sw;
 
+extern int opt_fading;
 extern int opt_fsize;
 extern int opt_fs;
 extern int opt_owntheme;
@@ -25,6 +31,7 @@ extern int opt_music;
 extern int opt_autosave;
 extern int opt_filter;
 extern int opt_kbd;
+extern int opt_justify;
 
 extern int opt_mode[2];
 extern char *opt_game;
