@@ -18,7 +18,9 @@ idf_t game_idf = NULL;
 
 int game_own_theme = 0;
 int game_theme_changed = 0;
+int mouse_filter_delay = 400;
 
+static int need_restart = 0;
 static int game_pic_w = 0;
 static int game_pic_h = 0;
 
@@ -91,8 +93,8 @@ static struct game *game_lookup(const char *name)
 
 int game_reset(void)
 {
+	free_last(); /* commit all user events */
 	game_release_theme();
-	free_last();
 	if (game_select(curgame_dir))
 		goto out;
 	if (game_apply_theme())
@@ -101,7 +103,7 @@ int game_reset(void)
 out:
 	game_done(0);
 	if (game_init(NULL)) {
-		game_error("");
+		game_error();
 		return -1;
 	}
 	return -1;
@@ -446,7 +448,7 @@ void game_menu(int nr)
 	game_menu_box(1, game_menu_gen());
 }
 
-int game_error(const char *name)
+int game_error(void)
 {
 	game_done(1);
 	if (game_init(NULL)) {
@@ -543,6 +545,7 @@ static int inv_enabled(void)
 
 int game_apply_theme(void)
 {
+	int align = game_theme.win_align;
 	layout_t lay;
 	textbox_t box;
 
@@ -551,7 +554,12 @@ int game_apply_theme(void)
 	if (!DIRECT_MODE)
 		game_clear(0, 0, game_theme.w, game_theme.h);
 	gfx_flip();
-	lay = txt_layout(game_theme.font, game_theme.win_align, game_theme.win_w, game_theme.win_h);
+	if (opt_justify == JUST_NO && align == ALIGN_JUSTIFY)
+		align = ALIGN_LEFT;
+	else if (opt_justify == JUST_YES && align == ALIGN_LEFT)
+		align = ALIGN_JUSTIFY;
+
+	lay = txt_layout(game_theme.font, align, game_theme.win_w, game_theme.win_h);
 	if (!lay)
 		return -1;
 	box = txt_box(game_theme.win_w, game_theme.win_h);
@@ -627,7 +635,7 @@ int game_restart(void)
 	game_save(-1);
 	game_done(0);
 	if (game_init(og)) {
-		game_error(og);
+		game_error();
 		return 0;
 	}
 	return 0;
@@ -682,18 +690,27 @@ int game_change_hz(int hz)
 	free_last_music();
 	snd_init(hz);
 	snd_volume_mus(cur_vol);
-	snd_free_wav(game_theme.click);
-	game_theme.click = snd_load_wav(game_theme.click_name);
 	sounds_reload();
 	game_music_player();
 	opt_hz = snd_hz();
 	return 0;
 }
 
-unsigned int	timer_counter = 0;
+unsigned long	timer_counter = 0;
 
 gtimer_t timer_han = NULL_TIMER;
 
+static void _game_gfx_commit(void *data)
+{
+	gfx_commit();
+}
+
+void game_gfx_commit(void)
+{
+	if (gfx_pending())
+		push_user_event(_game_gfx_commit, NULL);
+}
+
 static void anigif_do(void *data)
 {
 	void *v;
@@ -745,6 +762,7 @@ static void anigif_do(void *data)
 		}
 	}
 	game_cursor(CURSOR_ON);
+	game_gfx_commit();
 }
 
 int counter_fn(int interval, void *p)
@@ -808,11 +826,11 @@ int game_init(const char *name)
 	} else {
 		if (!game_load(-1)) /* tmp save */
 			goto out;
-		if (opt_autosave && !game_load(0))  /* autosave */
+		if ((opt_autosave & 1) && !game_load(0))  /* autosave */
 			goto out;
 		game_cmd("look", 0);
 		custom_theme_warn();
-		if (opt_autosave)
+		if (opt_autosave & 1)
 			game_save(0);
 	}
 out:
@@ -871,16 +889,16 @@ void game_done(int err)
 {
 	gfx_del_timer(timer_han);
 	timer_han = NULL_TIMER;
-
-	if (opt_autosave && curgame_dir && !err)
+	if ((opt_autosave & 1) && curgame_dir && !err)
 		game_save(0);
+	curgame_dir = NULL;
 	setdir(game_cwd);
 //	cfg_save();
 
 	if (menu_shown)
-		menu_toggle();
+		menu_toggle(-1);
+	free_last(); /* here all lost user callback are */
 	game_release_theme();
-	free_last();
 	game_theme_free();
 	input_clear();
 	snd_done();
@@ -888,9 +906,9 @@ void game_done(int err)
 //#ifndef ANDROID
 //	gfx_video_done();
 //#endif
-	curgame_dir = NULL;
 	game_own_theme = 0;
 	idf_done(game_idf); game_idf = NULL;
+	need_restart = 0;
 //	SDL_Quit();
 }	
 
@@ -978,14 +996,12 @@ int box_isscroll_up(int n)
 
 int box_isscroll_down(int n)
 {
-	layout_t l;
 	int off;
 	int h, hh;
 	if (el(n)->type != elt_box)
 		return -1;
 	el_size(n, NULL, &h);
-	l = txt_box_layout(el_box(n));
-	txt_layout_real_size(l, NULL, &hh);
+	txt_box_real_size(el_box(n), NULL, &hh);
 	off = txt_box_off(el_box(n));
 
 	if (hh - off > h)
@@ -998,7 +1014,6 @@ void box_update_scrollbar(int n)
 	struct el *elup = NULL;
 	struct el *eldown = NULL;
 //	struct el *elslide;
-	layout_t l;
 
 	int x1, y1;
 	int x2, y2;
@@ -1022,7 +1037,8 @@ void box_update_scrollbar(int n)
 		x2 = game_theme.inv_a_down_x;
 		y2 = game_theme.inv_a_down_y;
 //		elslide = el(el_islide);
-	}
+	} else /* impossible case */
+		return; 
 	if (!elup || !eldown)
 		return;	
 
@@ -1041,8 +1057,7 @@ void box_update_scrollbar(int n)
 	if (y2 == -1)
 		y2 = y1 + h - gfx_img_h(game_theme.a_down);
 
-	l = txt_box_layout(el_box(n));
-	txt_layout_real_size(l, NULL, &hh);
+	txt_box_real_size(el_box(n), NULL, &hh);
 	off = txt_box_off(el_box(n));
 
 	if (el_clear(elup->id)) {
@@ -1103,8 +1118,11 @@ img_t	game_pict_scale(img_t img, int ww, int hh)
 	game_pic_w = gfx_img_w(img);
 	game_pic_h = gfx_img_h(img);
 
-	if (!cache_have(gfx_image_cache(), img))
+	if (!cache_have(gfx_image_cache(), img)) {
+		game_pic_w = (int)((float)game_pic_w / (float)game_theme.scale);
+		game_pic_h = (int)((float)game_pic_h / (float)game_theme.scale);
 		return img; /* do not scale sprites! */
+	}
 
 	if (game_theme.scale > 1.0f)
 		theme_img_scale(&img);
@@ -1134,7 +1152,7 @@ img_t	game_pict_scale(img_t img, int ww, int hh)
 	}
 	if (scale < 0)
 		scale = 0;
-	img2 = gfx_scale(img, scale, scale);
+	img2 = gfx_scale(img, scale, scale, SCALABLE_THEME_SMOOTH);
 	gfx_free_image(img);
 	return img2;
 }
@@ -1315,6 +1333,17 @@ static void game_autosave(void)
 	}
 }
 
+static void game_instead_restart(void)
+{
+	int b;
+	if (!curgame_dir)
+		return;
+	instead_function("instead.get_restart", NULL);
+	b = instead_bretval(0); 
+	instead_clear();
+	need_restart = b;
+}
+
 static void finish_music(void *data)
 {
 	int rc;
@@ -1395,14 +1424,44 @@ typedef struct {
 	char	*fname;
 	wav_t	wav;
 	int	loaded;
+	int	system;
 } _snd_t;
-
+typedef struct {
+	_snd_t *snd;
+	int	loop;
+	int	channel;
+} _snd_req_t;
 static _snd_t *channels[SND_CHANNELS] = {};
+static _snd_req_t sound_reqs[SND_CHANNELS] = {};
+static void sound_play(_snd_t *sn, int chan, int loop);
+
+static void sound_callback(void *aux)
+{
+	_snd_req_t *r;
+	int channel = *((int *)aux);
+	int c = channel % SND_CHANNELS;
+	free(aux);
+//	fprintf(stderr, "finished: %d\n", channel);
+	channels[c] = NULL;
+	r = &sound_reqs[c];
+	if (r->snd) {
+		_snd_t *s = r->snd;
+		r->snd = NULL;
+		sound_play(s, channel, r->loop);
+	} else {
+		snd_halt_chan(channel, 0); /* to avoid races */
+	}
+}
 
 void game_channel_finished(int channel)
 {
-//	fprintf(stderr, "finished: %d\n", channel);
-	channels[channel % SND_CHANNELS] = NULL;
+	int *i = malloc(sizeof(channel));
+	if (!i) {
+		fprintf(stderr, "game_channel_finished: No memory\n");
+		return;
+	}
+	*i = channel;
+	push_user_event(sound_callback, i);
 }
 
 static int  sound_playing(_snd_t *snd)
@@ -1411,6 +1470,8 @@ static int  sound_playing(_snd_t *snd)
 	for (i = 0; i < SND_CHANNELS; i++) {
 		if (channels[i] == snd)
 			return i;
+		if (sound_reqs[i].snd == snd)
+			return i;
 	}
 	return -1;
 }
@@ -1418,9 +1479,21 @@ static int  sound_playing(_snd_t *snd)
 const char *sound_channel(int i)
 {
 	_snd_t *sn;
+	if (i >= SND_CHANNELS)
+		i = i % SND_CHANNELS;
+	if (i == -1) {
+		for (i = 0; i < SND_CHANNELS; i++) {
+			sn = channels[i];
+			if (sn && !sn->system)
+				return sn->fname;
+		}
+		return NULL;
+	}
 	sn = channels[i];
 	if (!sn)
 		return NULL;
+	if (sn->system)
+		return NULL; /* hidden system sound */
 	return sn->fname;
 }
 
@@ -1428,11 +1501,11 @@ static void sound_free(_snd_t *sn)
 {
 	if (!sn)
 		return;
+	list_del(&sn->list);
+	sounds_nr --;
 	free(sn->fname);
 	snd_free_wav(sn->wav);
-	list_del(&sn->list);
 	free(sn);
-	sounds_nr --;
 }
 
 static void sounds_shrink(void)
@@ -1457,16 +1530,27 @@ static void sounds_shrink(void)
 void sounds_free(void)
 {
 	int i = 0;
+	struct list_head *pos, *pos2;
+	_snd_t *sn;
+	pos = sounds.next;
+
 	snd_halt_chan(-1, 0); /* halt sounds */
-	while (!list_empty(&sounds)) {
-		_snd_t *sn = (_snd_t*)(sounds.next);
-//		if (sound_playing(sn) == -1) {
+	while (pos != &sounds) {
+		sn = (_snd_t*)pos;
+		pos2 = pos->next;
+		if (sn->system)
+			sn->loaded = 1; /* ref by system only */
+		else
 			sound_free(sn);
-//		}
+		pos = pos2;
 	}
-	for (i = 0; i < SND_CHANNELS; i++)
+	for (i = 0; i < SND_CHANNELS; i++) {
 		channels[i] = NULL;
-	sounds_nr = 0;
+		sound_reqs[i].snd = NULL;
+	}
+//	sounds_nr = 0;
+//	fprintf(stderr, "%d\n", sounds_nr);
+	input_uevents(); /* all callbacks */
 }
 
 static _snd_t *sound_find(const char *fname)
@@ -1483,9 +1567,36 @@ static _snd_t *sound_find(const char *fname)
 	return NULL;
 }
 
+static int sound_find_channel(int chan)
+{
+	int i;
+	for (i = 0; i < SND_CHANNELS; i ++) {
+		if (!channels[i] && !sound_reqs[i].snd)
+			return i;
+	}
+	return -1;
+}
+
 static void sound_play(_snd_t *sn, int chan, int loop)
 {
-	int c = snd_play(sn->wav, chan, loop - 1);
+	int c;
+	if (!sn)
+		return;
+	if (chan == -1) {
+		c = sound_find_channel(chan);
+		if (c == -1)
+			return; /* all channels are busy */
+	} else
+		c = chan;
+	if (channels[c]) {
+		sound_reqs[c].snd = sn;
+		sound_reqs[c].loop = loop;
+		sound_reqs[c].channel = chan;
+		snd_halt_chan(chan, 0); /* work in callback */
+		input_uevents(); /* all callbacks */
+		return;
+	}
+	c = snd_play(sn->wav, c, loop - 1);
 //	fprintf(stderr, "added: %d\n", c);
 	if (c == -1)
 		return;
@@ -1504,6 +1615,7 @@ static _snd_t *sound_add(const char *fname)
 	INIT_LIST_HEAD(&sn->list);
 	sn->fname = strdup(fname);
 	sn->loaded = 0;
+	sn->system = 0;
 	if (!sn->fname) {
 		free(sn);
 		return NULL;
@@ -1528,25 +1640,70 @@ static void sounds_reload(void)
 {
 	struct list_head *pos;
 	_snd_t *sn;
+	int i;
+	snd_halt_chan(-1, 0); /* stop all sound */
 	list_for_each(pos, &sounds) {
 		sn = (_snd_t*)pos;
 		snd_free_wav(sn->wav);
 		sn->wav = snd_load_wav(sn->fname);
 	}
+	for (i = 0; i < SND_CHANNELS; i++) {
+		channels[i] = NULL;
+		sound_reqs[i].snd = NULL;
+	}
+	input_clear(); /* all callbacks */
 }
 
-int sound_load(const char *fname)
+static void *_sound_get(const char *fname)
 {
 	_snd_t *sn;
 	sn = sound_find(fname);
 	if (sn) {
 		sn->loaded ++; /* to pin */
-		return 0;
+		return sn;
 	}
 	sn = sound_add(fname);
 	if (!sn)
-		return -1;
+		return NULL;
 	sn->loaded = 1;
+	return sn;
+}
+
+static void _sound_put(void *s)
+{
+	_snd_t *sn = (_snd_t *)s;
+	if (!sn || !sn->loaded)
+		return;
+	if (!sn->system || sn->loaded > 1)
+		sn->loaded --;
+	if (!sn->loaded && sound_playing(sn) == -1)
+		sound_free(sn);
+	return;
+}
+
+void *sound_get(const char *fname)
+{
+	_snd_t *sn = _sound_get(fname);
+	if (!sn)
+		return NULL;
+	sn->system = 1;
+	return sn;
+}
+
+void sound_put(void *s)
+{
+	_snd_t *sn = (_snd_t *)s;
+	if (!sn)
+		return;
+	sn->system = 0;
+	_sound_put(sn);
+}
+
+int sound_load(const char *fname)
+{
+	_snd_t *sn = _sound_get(fname);
+	if (!sn)
+		return -1;
 	return 0;
 }
 
@@ -1554,11 +1711,7 @@ void sound_unload(const char *fname)
 {
 	_snd_t *sn;
 	sn = sound_find(fname);
-	if (!sn || !sn->loaded)
-		return;
-	sn->loaded --;
-	if (!sn->loaded && sound_playing(sn) == -1)
-		sound_free(sn);
+	_sound_put(sn);
 	return;
 }
 
@@ -1673,26 +1826,41 @@ static int find_diff_pos(const char *p1, const char *p2)
 	return pos;
 }
 
-static void scroll_to_diff(const char *cmdstr, int cur_off)
+static void scroll_to_diff(const char *cmdstr, int cur_off, int new_scene)
 {
 	int off = 0;
 	int pos = 0;
 	int h = 0;
 	int hh = 0;
-	pos = find_diff_pos(cmdstr, last_cmd);
-	if (pos == -1)
-		off = cur_off;
-	else
-		off = txt_layout_pos2off(txt_box_layout(el_box(el_scene)), pos, &hh);
+
+	off = txt_layout_anchor(txt_box_layout(el_box(el_scene)), &hh); /* <a:#> tag? */
+	if (off == -1) {
+		if (new_scene)
+			return; /* nothing to do */
+		pos = find_diff_pos(cmdstr, last_cmd);
+		if (pos != -1)
+			off = txt_layout_pos2off(txt_box_layout(el_box(el_scene)), pos, &hh);
+		if (off == -1)
+			off = cur_off;
+	} 
+
 	el_size(el_scene, NULL, &h);
 
-	if (cur_off <= off && cur_off + h >= off + hh) { /* do not scroll */
+	if (game_theme.win_scroll_mode == 2 && (cur_off <= off && cur_off + h >= off + hh)) { /* do not scroll */
 		off = cur_off;
 	}
 
 	txt_box_scroll(el_box(el_scene), off);
 }
 
+static void scroll_to_last(void)
+{
+	int w, h;
+
+	txt_layout_size(txt_box_layout(el_box(el_scene)), &w, &h);
+	txt_box_scroll(el_box(el_scene), h);
+}
+
 int game_highlight(int x, int y, int on);
 
 int game_pict_modify(img_t p)
@@ -1710,7 +1878,7 @@ static void game_pict_clip(void)
 {
 	int x, y, w, h;
 
-	if (game_theme.gfx_mode == GFX_MODE_EMBEDDED) {
+	if (GFX_MODE(game_theme.gfx_mode) == GFX_MODE_EMBEDDED) {
 		el_clip(el_scene);
 		return;
 	}
@@ -1785,7 +1953,7 @@ int game_cmd(char *cmd, int click)
 	game_sound_player();
 
 	if (opt_click && click && !rc)
-		snd_play(game_theme.click, -1, 0);
+		sound_play(game_theme.click, -1, 1);
 
 	if (DIRECT_MODE) {
 		if (cmdstr)
@@ -1872,20 +2040,24 @@ int game_cmd(char *cmd, int click)
 		txt_layout_set(el_layout(el_title), NULL);
 
 	if (new_pict || fading || game_pict_modify(NULL) ||
-		(new_place && (game_theme.gfx_mode == GFX_MODE_FIXED))) {
+		(new_place && (GFX_MODE(game_theme.gfx_mode) == GFX_MODE_FIXED))) {
 		redraw_pict = 1;
 	}
 	game_pict_clip();
 
 	if (redraw_pict) {
 		if (el_img(el_spic)) {
-			if (game_theme.gfx_mode == GFX_MODE_EMBEDDED)
+			if (GFX_MODE(game_theme.gfx_mode) == GFX_MODE_EMBEDDED)
 				el_clear_nobg(el_spic);
 			else
 				el_clear(el_spic);
 			if (new_pict) {
 				gfx_free_image(el_img(el_spic));
 				el(el_spic)->p.p = NULL;
+				if (GFX_MODE(game_theme.gfx_mode) == GFX_MODE_EMBEDDED && 
+					!el(el_spic)->p.p) /* clear embedded gfx */
+				txt_layout_add_img(txt_box_layout(el_box(el_scene)), 
+					"scene", NULL);
 			}
 		}
 	}
@@ -1896,9 +2068,6 @@ int game_cmd(char *cmd, int click)
 
 		if (new_pict) {
 			img = gfx_load_image(pict);
-			if (el_img(el_spic))
-				gfx_free_image(el_img(el_spic));
-			el(el_spic)->p.p = NULL;
 			if (GFX_MODE(game_theme.gfx_mode) != GFX_MODE_FLOAT) 
 				img = game_pict_scale(img, game_theme.win_w, game_theme.max_scene_h);
 			else
@@ -1939,9 +2108,9 @@ int game_cmd(char *cmd, int click)
 	}
 	gfx_noclip();
 
-
 	/* clear area */
 	el_clear(el_ways);
+
 	el_clear(el_scene);
 
 	instead_function("instead.get_ways", NULL); 
@@ -1959,7 +2128,7 @@ int game_cmd(char *cmd, int click)
 		txt_layout_size(el_layout(el_ways), NULL, &ways_h);
 	} 
 	old_off = txt_box_off(el_box(el_scene));
-	if (game_theme.gfx_mode == GFX_MODE_EMBEDDED) {
+	if (GFX_MODE(game_theme.gfx_mode) == GFX_MODE_EMBEDDED) {
 		char *p;
 		pict_h = 0; /* to fake code bellow */
 		txt_layout_set(txt_box_layout(el_box(el_scene)), ""); /* hack, to null layout, but not images */
@@ -1990,10 +2159,6 @@ int game_cmd(char *cmd, int click)
 		txt_layout_set(txt_box_layout(el_box(el_scene)), cmdstr);
 		txt_box_set(el_box(el_scene), txt_box_layout(el_box(el_scene)));
 	}
-	if (!new_scene)
-		scroll_to_diff(cmdstr, old_off);
-	FREE(last_cmd);
-	last_cmd = cmdstr;
 	
 	el(el_ways)->y = el(el_title)->y + title_h + pict_h;
 
@@ -2021,8 +2186,16 @@ int game_cmd(char *cmd, int click)
 
 	txt_box_resize(el_box(el_scene), game_theme.win_w, game_theme.win_h - title_h - ways_h - pict_h);
 
+	if (game_theme.win_scroll_mode == 1 || game_theme.win_scroll_mode == 2) {
+		scroll_to_diff(cmdstr, old_off, new_scene);
+	} else if (game_theme.win_scroll_mode == 3) {
+		scroll_to_last();
+	}
+	FREE(last_cmd);
+	last_cmd = cmdstr;
+
 	el_draw(el_scene);
-	
+
 inv:
 	if (inv_enabled()) {
 		int off;
@@ -2067,8 +2240,9 @@ out:
 	gfx_flip();
 //	input_clear();
 err:
-	game_autosave();
 
+	game_autosave();
+	game_instead_restart();
 #if 0
 	if (err_msg) {
 		mouse_reset(1);
@@ -2180,10 +2354,18 @@ static struct el *click_el = NULL;
 static unsigned long click_time = 0;
 static int click_x = -1;
 static int click_y = -1;
+int menu_visible(void)
+{
+	if (menu_shown)
+		return cur_menu;
+	return 0;
+}
 
 int game_paused(void)
 {
-	return browse_dialog || menu_shown || use_xref || click_xref || gfx_fading() || minimized();
+	return browse_dialog || menu_shown || 
+		use_xref || click_xref || 
+		gfx_fading() || minimized() || instead_busy();
 }
 
 void menu_update(struct el *elem)
@@ -2236,12 +2418,14 @@ void mouse_reset(int hl)
 }
 
 
-void menu_toggle(void)
+void menu_toggle(int menu)
 {
 	mouse_reset(1);
 	menu_shown ^= 1;
 	if (!menu_shown)
 		cur_menu = menu_main;
+	else if (menu != -1)
+		cur_menu = menu;
 	game_menu_box(menu_shown, game_menu_gen());
 }
 
@@ -2282,9 +2466,9 @@ static int scroll_pdown(int id)
 int mouse_filter(int filter)
 {
 	static unsigned int old_counter = 0;
-	if (!opt_filter)
+	if (!opt_filter || !mouse_filter_delay)
 		return 0;
-	if (filter && (abs(old_counter - timer_counter) <= (400 / HZ))) /* 400 ms */
+	if (filter && (abs(old_counter - timer_counter) <= (mouse_filter_delay / HZ))) /* 400 ms */
 		return -1;
 	old_counter = timer_counter;
 	return 0;
@@ -2354,7 +2538,7 @@ int game_click(int x, int y, int action, int filter)
 	if (!xref) {
 		if (elem) {
 			if (elem->id == el_menu_button) {
-				menu_toggle();
+				menu_toggle(-1);
 			} else if (elem->id == el_sdown) {
 				scroll_pdown(el_scene);
 			} else if (elem->id == el_sup) {
@@ -2453,7 +2637,6 @@ void game_cursor(int on)
 	static img_t 	cur = NULL;
 	static int xc = 0, yc = 0, w = 0, h = 0; //, w, h;
 	int xx, yy, ww, hh;
-	
 	gfx_getclip(&xx, &yy, &ww, &hh);
 	gfx_noclip();
 	if (on == CURSOR_OFF)	
@@ -2475,7 +2658,6 @@ void game_cursor(int on)
 
 	if (on != CURSOR_DRAW)
 		cur = (use_xref) ? game_theme.use:game_theme.cursor;
-	
 	if (!cur) 
 		goto out;
 
@@ -3040,7 +3222,7 @@ int game_from_disk(void)
 		p = b;
 	game_done(0);
 	if (game_init(p)) {
-		game_error(p);
+		game_error();
 	}
 	return 0;
 #ifdef _USE_UNPACK
@@ -3053,19 +3235,22 @@ clean:
 #endif
 }
 #endif
+
+
 int game_loop(void)
 {
 	static int alt_pressed = 0;
 	static int shift_pressed = 0;
+	static int control_pressed = 0;
 	static int x = 0, y = 0;
 	struct inp_event ev;
 	memset(&ev, 0, sizeof(struct inp_event));
-
 	while (game_running) {
 		int rc;
 		ev.x = -1;
 //		game_cursor(CURSOR_CLEAR); /* release bg */
-		while ((rc = input(&ev, 1)) == AGAIN);
+		while (((rc = input(&ev, 1)) == AGAIN) && !need_restart)
+			gfx_commit();
 		if (rc == -1) {/* close */
 			break;
 		} else if (curgame_dir && (ev.type == KEY_DOWN || ev.type == KEY_UP)
@@ -3080,6 +3265,9 @@ int game_loop(void)
 		} else if (((ev.type ==  KEY_DOWN) || (ev.type == KEY_UP)) && 
 			(!is_key(&ev,"left shift") || !is_key(&ev, "right shift"))) {
 			shift_pressed = (ev.type == KEY_DOWN) ? 1:0;
+		} else if (((ev.type ==  KEY_DOWN) || (ev.type == KEY_UP)) &&
+			(!is_key(&ev,"left ctrl") || !is_key(&ev, "right ctrl"))) {
+			control_pressed = (ev.type == KEY_DOWN) ? 1:0;
 		} else if (ev.type == KEY_DOWN) {
 			if (!is_key(&ev, "escape")
 #if defined(S60) || defined(_WIN32_WCE)
@@ -3101,10 +3289,10 @@ int game_loop(void)
 				if (use_xref)
 					disable_use();
 				else	
-					menu_toggle();
+					menu_toggle(-1);
 			} else if (!is_key(&ev, "f1")) {
 				if (!menu_shown)
-					menu_toggle();
+					menu_toggle(-1);
 /*
 			} else if (!is_key(&ev, "f6")) {
 				mouse_reset(1);
@@ -3129,10 +3317,10 @@ int game_loop(void)
 			} else if (!is_key(&ev, "f5") && curgame_dir && !menu_shown) {
 				mouse_reset(1);
 				game_cmd("look", 0);
-			} else if (alt_pressed && !is_key(&ev, "r") && curgame_dir && !menu_shown && debug_sw) {
+			} else if ((alt_pressed || control_pressed) && !is_key(&ev, "r") && curgame_dir && !menu_shown && debug_sw) {
 				mouse_reset(1);
 				game_menu_act("/new");
-				shift_pressed = alt_pressed = 0;
+				shift_pressed = alt_pressed = control_pressed = 0;
 			} else if (!is_key(&ev, "f10")
 #ifdef ANDROID
 					|| ev.code == 270
@@ -3144,12 +3332,12 @@ int game_loop(void)
 				mouse_reset(1);
 				game_menu(menu_askquit);
 #endif
-			} else if (alt_pressed && (!is_key(&ev, "q") || !is_key(&ev, "f4"))) {
+			} else if ((alt_pressed | control_pressed) && (!is_key(&ev, "q") || !is_key(&ev, "f4"))) {
 				break;
 			} else if (alt_pressed &&
 				(!is_key(&ev, "enter") || !is_key(&ev, "return"))) {
 				int old_menu = (menu_shown) ? cur_menu: -1;
-				shift_pressed = alt_pressed = 0;
+				shift_pressed = alt_pressed = control_pressed = 0;
 				opt_fs ^= 1;
 				game_restart();
 				if (old_menu != -1)
@@ -3159,7 +3347,7 @@ int game_loop(void)
 #ifdef _USE_BROWSE
 				mouse_reset(1);
 				if (!game_from_disk()) {
-					shift_pressed = alt_pressed = 0;
+					shift_pressed = alt_pressed = control_pressed = 0;
 				}
 #endif
 #endif
@@ -3284,6 +3472,11 @@ int game_loop(void)
 		//	game_highlight(ev.x, ev.y, 1);
 		}
 
+		if (need_restart) {
+			need_restart = 0;
+			game_menu_act("/new");
+		}
+
 		if (!DIRECT_MODE || menu_shown) {
 			if (click_xref)
 				game_highlight(x, y, 1);
@@ -3293,12 +3486,12 @@ int game_loop(void)
 				game_highlight(x, y, 1);
 			}
 		}
-
 		game_cursor(CURSOR_ON);
 		if (err_msg) {
 			mouse_reset(1);
 			game_menu(menu_warning);
 		}
+		game_gfx_commit();
 	}
 	return 0;
 }
