@@ -7,6 +7,8 @@
 #define STEAD_PATH 	"./stead"
 #endif
 
+static int busy = 0;
+
 /* the Lua interpreter */
 static gtimer_t instead_timer = NULL_TIMER;
 static int instead_timer_nr = 0;
@@ -15,6 +17,11 @@ char 		*fromgame(const char *s);
 char 		*togame(const char *s);
 lua_State	*L = NULL;
 
+int instead_busy(void)
+{
+	return busy;
+}
+
 static int report (lua_State *L, int status) 
 {
 	if (status && !lua_isnil(L, -1)) {
@@ -74,7 +81,9 @@ static int docall (lua_State *L)
 		lua_pushcfunction(L, traceback);  /* push traceback function */
 		lua_insert(L, base);  /* put it under chunk and args */
 	}
+	busy ++;
 	status = lua_pcall(L, 0, LUA_MULTRET, base);
+	busy --;
 	if (debug_sw)
 		lua_remove(L, base);  /* remove traceback function */
 	/* force a complete garbage collection in case of errors */
@@ -258,7 +267,9 @@ int instead_function(char *s, struct instead_args *args)
 		lua_pushcfunction(L, traceback);  /* push traceback function */
 		lua_insert(L, base);  /* put it under chunk and args */
 	}
+	busy ++;
 	status = lua_pcall(L, method + n, LUA_MULTRET, base);
+	busy --;
 	if (debug_sw)
 		lua_remove(L, base);  /* remove traceback function */
 	if (status) {
@@ -493,8 +504,8 @@ static int luaB_is_sound(lua_State *L) {
 		c = -1;
 	else
 		c = atoi(chan);
-	r = snd_playing(c);
-	lua_pushboolean(L, (r != 0));  /* else not a number */
+	r = (sound_channel(c) != NULL);
+	lua_pushboolean(L, r);  /* else not a number */
 	return 1;
 }
 
@@ -514,8 +525,12 @@ static int luaB_get_gamepath(lua_State *L) {
 
 static int luaB_get_steadpath(lua_State *L) {
 	char stead_path[PATH_MAX];
-	strcpy(stead_path, game_cwd);
-	strcat(stead_path, "/");
+
+	if (STEAD_PATH[0] != '/') {
+		strcpy(stead_path, game_cwd);
+		strcat(stead_path, "/");
+	} else
+		stead_path[0] = 0;
 	strcat(stead_path, STEAD_PATH);
 	unix_path(stead_path);
 	lua_pushstring(L, stead_path);
@@ -524,8 +539,12 @@ static int luaB_get_steadpath(lua_State *L) {
 
 static int luaB_get_themespath(lua_State *L) {
 	char themes_path[PATH_MAX];
-	strcpy(themes_path, game_cwd);
-	strcat(themes_path, "/");
+
+	if (THEMES_PATH[0] != '/') {
+		strcpy(themes_path, game_cwd);
+		strcat(themes_path, "/");
+	} else
+		themes_path[0] = 0;
 	strcat(themes_path, THEMES_PATH);
 	unix_path(themes_path);
 	lua_pushstring(L, themes_path);
@@ -534,8 +553,11 @@ static int luaB_get_themespath(lua_State *L) {
 
 static int luaB_get_gamespath(lua_State *L) {
 	char games_path[PATH_MAX];
-	strcpy(games_path, game_cwd);
-	strcat(games_path, "/");
+	if (GAMES_PATH[0] != '/') {
+		strcpy(games_path, game_cwd);
+		strcat(games_path, "/");
+	} else
+		games_path[0] = 0;
 	strcat(games_path, GAMES_PATH);
 	unix_path(games_path);
 	lua_pushstring(L, games_path);
@@ -546,7 +568,7 @@ static int luaB_get_gamespath(lua_State *L) {
 static void instead_timer_do(void *data)
 {
 	char *p;
-	if (game_paused())
+	if (game_paused() || !curgame_dir)
 		goto out;
 	if (instead_function("stead.timer", NULL)) {
 		instead_clear();
@@ -608,8 +630,13 @@ static int luaB_theme_var(lua_State *L) {
 	game_own_theme = 1;
 	if (!opt_owntheme)
 		return 0;
-	if (!theme_setvar((char*)var, (char*)val))
-		game_theme_changed = 2;
+	if (!strcmp(var, "scr.w") ||
+		!strcmp(var, "scr.h")) /* filter resolution */
+		return 0;
+	if (!theme_setvar((char*)var, (char*)val)) {
+		if (strcmp(var, "win.scroll.mode")) /* let change scroll mode w/o theme reload */
+			game_theme_changed = 2;
+	}
 	return 0;
 }
 
@@ -962,7 +989,11 @@ static int luaB_sprite_size(lua_State *L) {
 	return 2;
 }
 
-static int luaB_draw_sprite(lua_State *L) {
+#define BLIT_COPY 0
+#define BLIT_DRAW 1
+#define BLIT_COMPOSE 2
+
+static int luaB_blit_sprite(lua_State *L, int mode) {
 	img_t s, d;
 	img_t img2 = NULL;
 	float v;
@@ -1013,59 +1044,21 @@ static int luaB_draw_sprite(lua_State *L) {
 			s = img2;
 	}
 	gfx_clip(game_theme.xoff, game_theme.yoff, game_theme.w - 2*game_theme.xoff, game_theme.h - 2*game_theme.yoff);
-	gfx_draw_from(s, x + xoff0, y + yoff0, w, h, d, xx + xoff, yy + yoff);
-	gfx_noclip();
-	gfx_free_image(img2);
-	lua_pushboolean(L, 1);
-	return 1;
-}
 
-static int luaB_copy_sprite(lua_State *L) {
-	img_t s, d;
-	img_t img2 = NULL;
-	float v;
-	const char *src = luaL_optstring(L, 1, NULL);
-	int x = luaL_optnumber(L, 2, 0);
-	int y = luaL_optnumber(L, 3, 0);
-	int w = luaL_optnumber(L, 4, -1);
-	int h = luaL_optnumber(L, 5, -1);
-	const char *dst = luaL_optstring(L, 6, NULL);
-	int xx = luaL_optnumber(L, 7, 0);
-	int yy = luaL_optnumber(L, 8, 0);
-	int xoff = 0, yoff = 0;
-	int xoff0 = 0, yoff0 = 0;
-	if (!src || !dst)
-		return 0;
-
-	s = grab_sprite(src, &xoff0, &yoff0);
-
-	d = grab_sprite(dst, &xoff, &yoff);	
-
-	if (!s || !d)
-		return 0;
-
-	v = game_theme.scale;
-
-	if (v != 1.0f) {
-		x *= v;
-		y *= v;
-		if (w != -1)
-			w = ceil(w * v);
-		if (h != -1)
-			h = ceil(h * v);
-		xx *= v;
-		yy *= v;
+	switch (mode) {
+	case BLIT_DRAW:
+		gfx_draw_from(s, x + xoff0, y + yoff0, w, h, d, xx + xoff, yy + yoff);
+		break;
+	case BLIT_COPY:
+		gfx_copy_from(s, x + xoff0, y + yoff0, w, h, d, xx + xoff, yy + yoff);
+		break;
+	case BLIT_COMPOSE:
+		gfx_compose_from(s, x + xoff0, y + yoff0, w, h, d, xx + xoff, yy + yoff);
+		break;
+	default:
+		break;
 	}
 
-	if (w == -1)
-		w = gfx_img_w(s) - 2 * xoff0;
-	if (h == -1)
-		h = gfx_img_h(s) - 2 * yoff0;
-
-	game_pict_modify(d);
-
-	gfx_clip(game_theme.xoff, game_theme.yoff, game_theme.w - game_theme.xoff * 2, game_theme.h - game_theme.yoff * 2);
-	gfx_copy_from(s, x + xoff0, y + yoff0, w, h, d, xx + xoff, yy + yoff);
 	gfx_noclip();
 	gfx_free_image(img2);
 	lua_pushboolean(L, 1);
@@ -1073,6 +1066,21 @@ static int luaB_copy_sprite(lua_State *L) {
 }
 
 
+static int luaB_draw_sprite(lua_State *L) 
+{
+	return luaB_blit_sprite(L, BLIT_DRAW);
+}
+
+static int luaB_copy_sprite(lua_State *L) 
+{
+	return luaB_blit_sprite(L, BLIT_COPY);
+}
+
+static int luaB_compose_sprite(lua_State *L) 
+{
+	return luaB_blit_sprite(L, BLIT_COMPOSE);
+}
+
 static int luaB_alpha_sprite(lua_State *L) {
 	_spr_t *sp;
 	img_t s;
@@ -1158,7 +1166,8 @@ static int luaB_scale_sprite(lua_State *L) {
 	const char *src = luaL_optstring(L, 1, NULL);
 	float xs = luaL_optnumber(L, 2, 0);
 	float ys = luaL_optnumber(L, 3, 0);
-	const char *desc = luaL_optstring(L, 4, NULL);
+	int smooth = lua_toboolean(L, 4);
+	const char *desc = luaL_optstring(L, 5, NULL);
 
 	if (!src)
 		return 0;
@@ -1173,7 +1182,7 @@ static int luaB_scale_sprite(lua_State *L) {
 	if (ys == 0)
 		ys = xs;
 
-	img2 = gfx_scale(s, xs, ys);
+	img2 = gfx_scale(s, xs, ys, smooth);
 
 	if (!img2)
 		return 0;
@@ -1204,7 +1213,8 @@ static int luaB_rotate_sprite(lua_State *L) {
 
 	const char *src = luaL_optstring(L, 1, NULL);
 	float angle = luaL_optnumber(L, 2, 1.0f);
-	const char *desc = luaL_optstring(L, 3, NULL);
+	int smooth = lua_toboolean(L, 3);
+	const char *desc = luaL_optstring(L, 4, NULL);
 
 	if (!src)
 		return 0;
@@ -1212,8 +1222,7 @@ static int luaB_rotate_sprite(lua_State *L) {
 	s = cache_lookup(gfx_image_cache(), src);
 	if (!s)
 		return 0;
-	
-	img2 = gfx_rotate(s, angle);
+	img2 = gfx_rotate(s, angle, smooth);
 
 	if (!img2)
 		return 0;
@@ -1358,7 +1367,42 @@ static int luaB_free_sprite(lua_State *L) {
 }
 
 static int luaB_show_menu(lua_State *L) {
-	menu_toggle();
+	const char *menu = luaL_optstring(L, 1, NULL);
+	if (menu) {
+		if (!strcmp(menu, "save"))
+			menu_toggle(menu_save);
+		else if (!strcmp(menu, "load"))
+			menu_toggle(menu_load);
+		else if (!strcmp(menu, "quit") || !strcmp(menu, "exit"))
+			menu_toggle(menu_askquit);
+		else
+			menu_toggle(-1);
+		return 0;
+	}
+	menu_toggle(menu_main);
+	return 0;
+}
+
+static int luaB_stead_busy(lua_State *L) {
+	static unsigned long busy_time = 0;
+	int busy = lua_toboolean(L, 1);
+	if (busy) {
+		struct inp_event ev;
+		memset(&ev, 0, sizeof(ev));
+		while (input(&ev, 0) == AGAIN);
+		if (ev.type == MOUSE_MOTION)
+			game_cursor(CURSOR_ON); /* to make all happy */
+		if (!busy_time)
+			busy_time = gfx_ticks();
+		if (gfx_ticks() - busy_time >= 750 && menu_visible() != menu_wait)
+			game_menu(menu_wait);
+		game_gfx_commit();
+		return 0;
+	}
+	if (menu_visible() == menu_wait) {
+		menu_toggle(-1);
+	}
+	busy_time = 0;
 	return 0;
 }
 
@@ -1450,6 +1494,17 @@ static int luaB_mouse_pos(lua_State *L) {
 	return 2;
 }
 
+extern int mouse_filter_delay;
+
+static int luaB_mouse_filter(lua_State *L) {
+	int d = luaL_optnumber(L, 1, -1);
+	int ov = mouse_filter_delay;
+	if (d != -1)
+		mouse_filter_delay = d;
+	lua_pushnumber(L, ov);
+	return 1;
+}
+
 static int luaB_get_ticks(lua_State *L) {
 	lua_pushnumber(L, gfx_ticks());
 	return 1;
@@ -1601,45 +1656,49 @@ static const luaL_Reg base_funcs[] = {
 	{"doencfile", luaB_doencfile},
 	{"dofile", luaB_dofile},
 	{"print", luaB_print}, /* for some mystic, it is needed in win version (with -debug) */
-	{"is_sound", luaB_is_sound},
-	{"get_savepath", luaB_get_savepath},
-	{"get_gamepath", luaB_get_gamepath},
-	{"get_steadpath", luaB_get_steadpath},
-	{"get_themespath", luaB_get_themespath},
-	{"get_gamespath", luaB_get_gamespath},
-	{"set_timer", luaB_set_timer},
-	{"theme_var", luaB_theme_var},
-	{"theme_name", luaB_theme_name},
-	{"readdir", dir_iter_factory},
-	{"menu_toggle", luaB_show_menu},
+	{"table_get_maxn", luaB_maxn},
 
-	{"sound_load", luaB_load_sound},
-	{"sound_free", luaB_free_sound},
-	{"sound_channel", luaB_channel_sound},
-	{"sound_panning", luaB_panning_sound},
-	{"sound_volume", luaB_volume_sound},
-	{"sounds_free", luaB_free_sounds},
-	
-	{"mouse_pos", luaB_mouse_pos},
+	{"instead_readdir", dir_iter_factory},
+	{"instead_sound", luaB_is_sound},
+	{"instead_savepath", luaB_get_savepath},
+	{"instead_gamepath", luaB_get_gamepath},
+	{"instead_steadpath", luaB_get_steadpath},
+	{"instead_themespath", luaB_get_themespath},
+	{"instead_gamespath", luaB_get_gamespath},
+	{"instead_timer", luaB_set_timer},
+	{"instead_theme_var", luaB_theme_var},
+	{"instead_theme_name", luaB_theme_name},
+	{"instead_menu_toggle", luaB_show_menu},
+	{"instead_busy", luaB_stead_busy},
+	{"instead_sound_load", luaB_load_sound},
+	{"instead_sound_free", luaB_free_sound},
+	{"instead_sound_channel", luaB_channel_sound},
+	{"instead_sound_panning", luaB_panning_sound},
+	{"instead_sound_volume", luaB_volume_sound},
+	{"instead_sounds_free", luaB_free_sounds},
 
-	{"font_load", luaB_load_font},
-	{"font_free", luaB_free_font},
-	{"font_scaled_size", luaB_font_size_scaled},
-	{"get_ticks", luaB_get_ticks},
-	{"sprite_load", luaB_load_sprite},
-	{"sprite_text", luaB_text_sprite},
-	{"sprite_free", luaB_free_sprite},
-	{"sprites_free", luaB_free_sprites},
-	{"sprite_draw", luaB_draw_sprite},
-	{"sprite_copy", luaB_copy_sprite},
-	{"sprite_fill", luaB_fill_sprite},
-	{"sprite_dup", luaB_dup_sprite},
-	{"sprite_alpha", luaB_alpha_sprite},
-	{"sprite_size", luaB_sprite_size},
-	{"sprite_scale", luaB_scale_sprite},
-	{"sprite_rotate", luaB_rotate_sprite},
-	{"sprite_text_size", luaB_text_size},
-	{"sprite_pixel", luaB_pixel_sprite},
+	{"instead_mouse_pos", luaB_mouse_pos},
+	{"instead_mouse_filter", luaB_mouse_filter},
+
+	{"instead_font_load", luaB_load_font},
+	{"instead_font_free", luaB_free_font},
+	{"instead_font_scaled_size", luaB_font_size_scaled},
+	{"instead_ticks", luaB_get_ticks},
+	{"instead_sprite_load", luaB_load_sprite},
+	{"instead_sprite_text", luaB_text_sprite},
+	{"instead_sprite_free", luaB_free_sprite},
+	{"instead_sprites_free", luaB_free_sprites},
+	{"instead_sprite_draw", luaB_draw_sprite},
+	{"instead_sprite_copy", luaB_copy_sprite},
+	{"instead_sprite_compose", luaB_compose_sprite},
+	{"instead_sprite_fill", luaB_fill_sprite},
+	{"instead_sprite_dup", luaB_dup_sprite},
+	{"instead_sprite_alpha", luaB_alpha_sprite},
+	{"instead_sprite_size", luaB_sprite_size},
+	{"instead_sprite_scale", luaB_scale_sprite},
+	{"instead_sprite_rotate", luaB_rotate_sprite},
+	{"instead_sprite_text_size", luaB_text_size},
+	{"instead_sprite_pixel", luaB_pixel_sprite},
 	
 	{"bit_or", luaB_bit_or},
 	{"bit_and", luaB_bit_and},
@@ -1657,7 +1716,6 @@ static const luaL_Reg base_funcs[] = {
 	{"bit_signed", luaB_bit_signed},
 	{"bit_unsigned", luaB_bit_unsigned},
 
-	{"get_table_maxn", luaB_maxn},
 	{NULL, NULL}
 };
 
@@ -1714,7 +1772,7 @@ static int instead_platform(void)
 static int instead_package(const char *path)
 {
 	char *p;
-#ifdef _WIN32_WCE
+#if defined(_WIN32_WCE)
 	char stead_path[PATH_MAX * 4] = "package.path=\""; /* wince have not cwd :) */
 	if (path) {
 		strcat(stead_path, path);
@@ -1746,8 +1804,10 @@ static int instead_package(const char *path)
 /*	putenv(stead_path); */
 	return 0;
 }
+
 int instead_init(const char *path)
 {
+	busy = 0;
 	setlocale(LC_ALL,"");
 	setlocale(LC_NUMERIC,"C"); /* to avoid . -> , in numbers */	
 //	strcpy(curcp, "UTF-8");
