@@ -8,7 +8,7 @@ static int themes_menu_from = 0;
 
 static int cur_lang = 0;
 
-int cur_menu = 0;
+int cur_menu = menu_main;
 
 char *UNKNOWN_ERROR = NULL;
 char *ERROR_MENU = NULL;
@@ -26,6 +26,7 @@ char *SETTINGS_GFX_MENU = NULL;
 char *SETTINGS_OTH_MENU = NULL;
 char *CUSTOM_THEME_MENU = NULL;
 char *OWN_THEME_MENU = NULL;
+char *WAIT_MENU = NULL;
 char *SELECT_GAME_MENU = NULL;
 char *SELECT_THEME_MENU = NULL;
 char *SAVED_MENU = NULL;
@@ -212,7 +213,10 @@ static int pages_menu(char *res, int nr, int max, const char *menu, const char *
 
 static void games_menu(void)
 {
-	int i, n, append_browse = 0;
+	int i, n;
+#ifdef _USE_BROWSE
+	int append_browse = 0;
+#endif
 	char tmp[PATH_MAX];
 #ifdef _USE_BROWSE
 	snprintf(tmp, sizeof(tmp), " <u><a:/browse>%s</a></u>\n", BROWSE_MENU);
@@ -222,8 +226,10 @@ static void games_menu(void)
 	strcpy(menu_buff, SELECT_GAME_MENU);
 	if ((games_nr - 1) / MENU_GAMES_MAX)
 		pages_menu(menu_buff, games_menu_from / MENU_GAMES_MAX, (games_nr - 1) / MENU_GAMES_MAX + 1, "games", tmp);
+#ifdef _USE_BROWSE
 	else 
 		append_browse = 1;
+#endif
 	for (i = games_menu_from, n = 0; i < games_nr && n < MENU_GAMES_MAX; i ++) {
 		
 		if (!games[i].name[0]) /* empty */
@@ -342,12 +348,15 @@ char *game_menu_gen(void)
 	} else if (cur_menu == menu_about) {
 		snprintf(menu_buff, sizeof(menu_buff), ABOUT_MENU, VERSION);
 	} else if (cur_menu == menu_settings) {
+		char *just[JUST_MAX] = { FROM_THEME, OFF, ON };
 		char *kbd [KBD_MAX] = { KBD_MODE_SMART, KBD_MODE_LINKS, KBD_MODE_SCROLL };
-
+		opt_kbd = (unsigned int)opt_kbd % KBD_MAX;
+		opt_justify = (unsigned int)opt_justify % JUST_MAX;
 		switch (menu_settings_num) {
 		case 0:
 			snprintf(menu_buff, sizeof(menu_buff), SETTINGS_GFX_MENU, 
-			opt_get_mode(), opt_fs?ON:OFF, opt_fsize, opt_hl?ON:OFF, opt_owntheme?ON:OFF);
+			opt_get_mode(), opt_fs?ON:OFF, opt_fsize, just[opt_justify],
+				opt_hl?ON:OFF, opt_fading?ON:OFF, opt_owntheme?ON:OFF);
 			break;
 		case 1:
 			snprintf(menu_buff, sizeof(menu_buff), SETTINGS_SND_MENU, 
@@ -356,7 +365,7 @@ char *game_menu_gen(void)
 		case 2:
 			snprintf(menu_buff, sizeof(menu_buff), SETTINGS_OTH_MENU, 
 			opt_motion?ON:OFF, opt_filter?ON:OFF, kbd[opt_kbd],
-			langs[cur_lang].name,  opt_autosave?ON:OFF);
+			langs[cur_lang].name,  (opt_autosave & 1)?ON:OFF);
 			break;
 		}
 	} else if (cur_menu == menu_askquit) {
@@ -369,6 +378,8 @@ char *game_menu_gen(void)
 		themes_menu();
 	} else if (cur_menu == menu_own_theme) {
 		strcpy(menu_buff, OWN_THEME_MENU);
+	} else if (cur_menu == menu_wait) {
+		strcpy(menu_buff, WAIT_MENU);
 	} else if (cur_menu == menu_custom_theme) {
 		strcpy(menu_buff, CUSTOM_THEME_MENU);
 	} else if (cur_menu == menu_load) {
@@ -395,7 +406,7 @@ int game_menu_act(const char *a)
 	static int old_vol = 0;
 
 	if (!strcmp(a, "/autosave")) {
-		opt_autosave ^= 1;
+		opt_autosave = !(opt_autosave & 1);
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/kbd")) {
 		opt_kbd += 1;
@@ -403,7 +414,7 @@ int game_menu_act(const char *a)
 			opt_kbd = 0;
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/owntheme")) {
-		opt_owntheme ^= 1;
+		opt_owntheme = !opt_owntheme;
 		if (game_own_theme)
 			restart_needed = 1;
 		game_menu_box(1, game_menu_gen());
@@ -420,18 +431,28 @@ int game_menu_act(const char *a)
 #ifndef ANDROID
 		if (gfx_next_mode(&opt_mode[0], &opt_mode[1]))
 			opt_mode[0] = opt_mode[1] = -1;
-		if (SCALABLE_THEME)
-			restart_needed = 1;
+		restart_needed = 1;
 		game_menu_box(1, game_menu_gen());
 #endif
 	} else if (!strcmp(a, "/mode--")) {	
 #ifndef ANDROID
 		if (gfx_prev_mode(&opt_mode[0], &opt_mode[1]))
 			opt_mode[0] = opt_mode[1] = -1;
-		if (SCALABLE_THEME)
-			restart_needed = 1;
+		restart_needed = 1;
 		game_menu_box(1, game_menu_gen());
 #endif
+	} else if (!strcmp(a, "/just++")) {
+		restart_needed = 1;
+		opt_justify ++;
+		if (opt_justify >= JUST_MAX)
+			opt_justify = 0;
+		game_menu_box(1, game_menu_gen());
+	} else if (!strcmp(a, "/just--")) {	
+		restart_needed = 1;
+		opt_justify --;
+		if (opt_justify < 0)
+			opt_justify = JUST_MAX - 1;
+		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/fs--")) {
 		opt_fsize --;
 		if (FONT_SZ(game_theme.font_size) > FONT_MIN_SZ * game_theme.scale) {
@@ -449,6 +470,9 @@ int game_menu_act(const char *a)
 	} else if (!strcmp(a, "/hl")) {
 		opt_hl ^= 1;
 		game_menu_box(1, game_menu_gen());
+	} else if (!strcmp(a, "/fading")) {
+		opt_fading ^= 1;
+		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/fs")) {
 #ifndef ANDROID
 		restart_needed = 1;
@@ -488,7 +512,7 @@ int game_menu_act(const char *a)
 		if (curgame_dir && !strcmp(curgame_dir, games[gtr].dir)) {
 			game_done(0);
 			if (game_init(NULL)) {
-				game_error("");
+				game_error();
 				return 0;
 			}
 		}
@@ -529,7 +553,7 @@ int game_menu_act(const char *a)
 
 /* remove autlosave */
 		s = game_save_path(0, 0);
-		if (s && !access(s, R_OK) && opt_autosave)
+		if (s && !access(s, R_OK) && (opt_autosave & 1))
 			unlink (s);
 		game_menu_box(0, NULL);
 		if (!game_reset()) {
@@ -647,7 +671,7 @@ int game_menu_act(const char *a)
 		if (p) {
 			game_done(0);
 			if (game_init(p)) {
-				game_error(p);
+				game_error();
 			}
 			free(p);
 		}
@@ -661,7 +685,7 @@ int game_menu_act(const char *a)
 			game_save(-1);
 			game_done(0);
 			if (game_init(og)) {
-				game_error(og);
+				game_error();
 			} else if (curgame_dir && game_own_theme && opt_owntheme) {
 				game_menu(menu_own_theme);
 			}
@@ -705,6 +729,7 @@ static void lang_free(void)
 	FREE(SETTINGS_OTH_MENU);
 	FREE(CUSTOM_THEME_MENU);
 	FREE(OWN_THEME_MENU);
+	FREE(WAIT_MENU);
 	FREE(SELECT_GAME_MENU);
 	FREE(SELECT_THEME_MENU);
 	FREE(SAVED_MENU);
@@ -728,7 +753,7 @@ static int lang_ok(void)
 	if (UNKNOWN_ERROR && ERROR_MENU && WARNING_MENU && SAVE_SLOT_EMPTY &&
 		SELECT_LOAD_MENU && AUTOSAVE_SLOT && BROKEN_SLOT && SELECT_SAVE_MENU &&
 		MAIN_MENU && ABOUT_MENU && BACK_MENU && SETTINGS_SND_MENU && SETTINGS_GFX_MENU && SETTINGS_OTH_MENU &&
-		CUSTOM_THEME_MENU && OWN_THEME_MENU && SELECT_GAME_MENU && SELECT_THEME_MENU &&
+		CUSTOM_THEME_MENU && OWN_THEME_MENU && SELECT_GAME_MENU && SELECT_THEME_MENU && WAIT_MENU &&
 		SAVED_MENU && NOGAMES_MENU && NOTHEMES_MENU && QUIT_MENU && REMOVE_MENU &&
 		ON && OFF && KBD_MODE_LINKS && KBD_MODE_SMART && KBD_MODE_SCROLL && CANCEL_MENU &&
 		FROM_THEME && DISABLED_SAVE_MENU && BROWSE_MENU)
@@ -753,6 +778,7 @@ struct parser lang_parser[] = {
 	{ "SETTINGS_OTH_MENU", parse_esc_string, &SETTINGS_OTH_MENU },
 	{ "CUSTOM_THEME_MENU", parse_esc_string, &CUSTOM_THEME_MENU },
 	{ "OWN_THEME_MENU", parse_esc_string, &OWN_THEME_MENU },
+	{ "WAIT_MENU", parse_esc_string, &WAIT_MENU },
 	{ "SELECT_GAME_MENU", parse_esc_string, &SELECT_GAME_MENU },
 	{ "SELECT_THEME_MENU", parse_esc_string, &SELECT_THEME_MENU },
 	{ "SAVED_MENU", parse_esc_string, &SAVED_MENU },
