@@ -210,6 +210,8 @@ struct parser cmd_parser[] = {
 	{ "win.y", parse_int, &game_theme.win_y },
 	{ "win.w", parse_int, &game_theme.win_w },
 	{ "win.h", parse_int, &game_theme.win_h },
+	{ "win.scroll.mode", parse_int, &game_theme.win_scroll_mode },
+
 	{ "win.fnt.name", parse_full_path, &game_theme.font_name, CHANGED_FONT },
 	{ "win.fnt.size", parse_int, &game_theme.font_size, CHANGED_FONT },
 	{ "win.fnt.height", parse_float, &game_theme.font_height },
@@ -354,6 +356,7 @@ struct game_theme game_theme = {
 	.menu_button_name = NULL,
 	.menu_button = NULL,
 	.win_align = ALIGN_JUSTIFY,
+	.win_scroll_mode = 2,
 	.gfx_mode = GFX_MODE_EMBEDDED,
 	.inv_mode = INV_MODE_VERT | INV_ALIGN_SET(ALIGN_LEFT),
 	.click_name = NULL,
@@ -403,16 +406,19 @@ int game_theme_free(void)
 
 	if (game_theme.use)
 		gfx_free_image(game_theme.use);
-
+	if (game_theme.cursor) {
+		gfx_free_image(game_theme.cursor);
+		gfx_set_cursor(NULL, 0, 0);
+	}
 	if (game_theme.bg)
 		gfx_free_image(game_theme.bg);
 
 	if (game_theme.menu_button)
 		gfx_free_image(game_theme.menu_button);
 
-	if (game_theme.click)
-		snd_free_wav(game_theme.click);
-
+	if (game_theme.click) {
+		sound_put(game_theme.click);
+	}
 	game_theme.font = game_theme.inv_font = game_theme.menu_font = NULL;
 	game_theme.a_up = game_theme.a_down = game_theme.use = NULL;
 	game_theme.inv_a_up = game_theme.inv_a_down = NULL;
@@ -420,6 +426,7 @@ int game_theme_free(void)
 	game_theme.bg = NULL;
 	game_theme.click = NULL;
 	game_theme.cur_x = game_theme.cur_y = 0;
+	game_theme.cursor = game_theme.use = NULL;
 	return 0;
 }
 
@@ -433,13 +440,14 @@ int theme_img_scale(img_t *p)
 	if (!cache_have(gfx_image_cache(), *p))
 		return 0; /* do not scale sprites! */
 
-	pic = gfx_scale(*p, v, v); 
+	pic = gfx_scale(*p, v, v, SCALABLE_THEME_SMOOTH);
 	if (!pic)
 		return -1;
 	gfx_free_image(*p); 
 	*p = pic;
 	return 0;
 }
+
 static  int game_theme_scale(int w, int h)
 {
 	int i;
@@ -455,11 +463,29 @@ static  int game_theme_scale(int w, int h)
 		h = t->h;
 		goto out;
 	}
+
+
 	xs = (float)w / (float)t->w;
 	ys = (float)h / (float)t->h;
 	
 	v = (xs < ys)?xs:ys;
 
+	if (!SCALABLE_THEME) {
+		if (v > 1.0f) {
+			int ff = 1;
+			while (ff && ff <= v && ff < 0x1000)
+				ff <<= 1;
+			ff >>= 1;
+			v = ff;
+		} else {
+			float f = ceil(1.0f / v);
+			int ff = 1;
+			while (ff && ff < f && ff < 0x1000)
+				ff <<= 1;
+			v = 1.0f / (float)ff;
+		}
+	}
+
 	xoff = (w - t->w*v)/2;
 	yoff = (h - t->h*v)/2;
 
@@ -621,8 +647,10 @@ int game_theme_optimize(void)
 /* todo: check errors */
 	struct game_theme *t = &game_theme;
 
-	if (t->bg && cache_have(gfx_image_cache(), t->bg))
+	if (t->bg && cache_have(gfx_image_cache(), t->bg)) {
 		t->bg = gfx_display_alpha(t->bg);
+		gfx_unset_alpha(t->bg);
+	}
 	if (t->a_up && cache_have(gfx_image_cache(), t->a_up))
 		t->a_up = gfx_display_alpha(t->a_up);
 	if (t->a_down && cache_have(gfx_image_cache(), t->a_down))
@@ -633,8 +661,10 @@ int game_theme_optimize(void)
 		t->inv_a_down = gfx_display_alpha(t->inv_a_down);
 	if (t->use && cache_have(gfx_image_cache(), t->use))
 		t->use = gfx_display_alpha(t->use);
-	if (t->cursor && cache_have(gfx_image_cache(), t->cursor))
+	if (t->cursor && cache_have(gfx_image_cache(), t->cursor)) {
 		t->cursor = gfx_display_alpha(t->cursor);
+		gfx_set_cursor(t->cursor, t->cur_x, t->cur_y);
+	}
 	if (t->menu_button && cache_have(gfx_image_cache(), t->menu_button))
 		t->menu_button = gfx_display_alpha(t->menu_button);
 	return 0;
@@ -704,13 +734,13 @@ static int game_theme_update_data(void)
 		gfx_free_image(t->bg);
 		t->bg = NULL;
 		if (t->bg_name[0] && !(t->bg = gfx_load_image(t->bg_name)))
-			goto err;
+			goto skip; /* not fatal */
 		if (theme_img_scale(&t->bg))
 			goto err;
 		if (theme_bg_scale())
 			goto err;
 	}
-
+skip:
 	if (t->use_name && (t->changed & CHANGED_USE)) {
 		gfx_free_image(t->use);	
 		if (!(t->use = gfx_load_image(t->use_name)))
@@ -725,6 +755,7 @@ static int game_theme_update_data(void)
 			goto err;
 		if (theme_img_scale(&t->cursor))
 			goto err;
+		gfx_set_cursor(t->cursor, t->cur_x, t->cur_y);
 	}
 	
 	if (t->menu_button_name && (t->changed & CHANGED_BUTTON)) {
@@ -736,8 +767,8 @@ static int game_theme_update_data(void)
 	}
 	
 	if (t->click_name && (t->changed & CHANGED_CLICK)) {
-		snd_free_wav(t->click);
-		t->click = snd_load_wav(t->click_name);
+		sound_put(t->click);
+		t->click = sound_get(t->click_name);
 	}
 
 //	free_theme_strings(); /* todo, font */
@@ -760,11 +791,13 @@ int game_theme_update(void)
 	game_release_theme();
 	if (game_theme_update_data()) {
 		fprintf(stderr, "Can not update theme!\n");
+		game_error();
 		return -1;
 	}
 
 	if (game_apply_theme()) {
 		fprintf(stderr, "Can not apply theme!\n");
+		game_error();
 		return -1;
 	}
 	return 0;
@@ -776,15 +809,26 @@ int game_theme_init(void)
 	int w  = opt_mode[0];
 	int h  = opt_mode[1];
 
-	if ((w == -1) 
-		&& (gfx_get_max_mode(&w, &h) || (game_theme.w <= w && game_theme.h <= h))) {
-		w = opt_mode[0];
-		h = opt_mode[1];
+	if (w == -1) { /* as theme */
+		if (gfx_get_max_mode(&w, &h) || (game_theme.w <= w && game_theme.h <= h)) {
+			w = opt_mode[0];
+			h = opt_mode[1];
+		}
 	}
-	if (!SCALABLE_THEME) { /* no scalable? TODO: message? */
+#if 0 /* not useful */
+	 else if (opt_fs) { /* selected and full screen */
+		if (gfx_get_max_mode(&w, &h) || (opt_mode[0] <= w && opt_mode[1] <= h)) {
+			w = opt_mode[0];
+			h = opt_mode[1];
+		}
+	}
+#endif
+#if 0
+	if (!SCALABLE_THEME && (w < game_theme.w || h < game_theme.h)) { /* no scalable? TODO: message? */
 		w = game_theme.w;
 		h = game_theme.h;
 	}
+#endif
 	game_theme_scale(w, h);
 
 	if (gfx_set_mode(game_theme.w, game_theme.h, opt_fs)) {
