@@ -1,8 +1,8 @@
 game.action = stead.hook(game.action, function(f, s, cmd, ...)
 	if cmd == 'user_kbd' then
 		local r,v;
-		if here().kbd then
-			r,v = stead.call(here(), 'kbd', 
+		if stead.here().kbd then
+			r,v = stead.call(stead.here(), 'kbd', 
 			input.key_event.down, input.key_event.key);
 		elseif s.kbd then
 			r,v = stead.call(s, 'kbd', 
@@ -27,7 +27,7 @@ stead.module_init(function()
 	input._key_hooks = {}
 end)
 
-function hook_keys(...)
+stead.hook_keys = function(...)
 	local i
 	local a = {...};
 	for i = 1, stead.table.maxn(a) do
@@ -35,7 +35,7 @@ function hook_keys(...)
 	end
 end
 
-function unhook_keys(...)
+stead.unhook_keys = function(...)
 	local i
 	local a = {...};
 	for i = 1, stead.table.maxn(a) do
@@ -43,4 +43,6 @@ function unhook_keys(...)
 	end
 end
 
+hook_keys = stead.hook_keys
+unhook_keys = stead.unhook_keys
 -- vim:ts=4
